﻿using AsReaderP3xUAndroidDemo.Platforms.Android;
using Com.Asreader.P3xu;
using Com.Asreader.P3xu.Device.Type;
using static Android.Media.TV.TvInputService;

namespace AsReaderP3xUAndroidDemo;

public partial class MainPage : ContentPage
{
    private AsReaderP3xUMAUIManager asReaderP3xUMAUIManager;
    private AsReaderP3xUCallback asReaderP3xUCallback;
    private string appVersion;
    private string dllVereison;
    private string sdkVersion;
    private string firmwareVersion;
    private string rfModuleVersion;
    private string serialVersion;
    private string globalBandVersion;
    private string HardwareVersion;

    public MainPage()
    {
        InitializeComponent();
        ConnectButton.Text = "Connect";
        MainActivityIndicator.IsRunning = false;
        asReaderP3xUMAUIManager = AsReaderP3xUMAUIManager.sharedInstance();
        asReaderP3xUCallback = AsReaderP3xUCallback.sharedInstance();
    }
    protected override void OnAppearing()
    {
        base.OnAppearing();
        if (asReaderP3xUMAUIManager.isConnect == true)
        {
            reloadUI(true);
        }
        else
        {
            reloadUI(false);
        }
        asReaderP3xUCallback.OnStateChanged = (AsReaderP3xU p0, AsReaderP3xUConnectionState p1) =>
        {
            OnStateChanged(p0, p1);
        };
        asReaderP3xUCallback.OnKeyEvent = (AsReaderP3xU p0, AsReaderP3xUKeyType p1, AsReaderP3xUKeyState p2) =>
        {
            return false;
        };
    }
    private void OnStateChanged(AsReaderP3xU p0, AsReaderP3xUConnectionState p1)
    {
        if (p1 == AsReaderP3xUConnectionState.Connected)
        {
            reloadUI(true);
        }
        else if (p1 == AsReaderP3xUConnectionState.Connecting)
        {
            StartActivityIndicator();
        }
        else if (p1 == AsReaderP3xUConnectionState.Disconnected)
        {
            reloadUI(false);
        }
    }
    private void reloadUI(bool status)
    {
        if (status == true)
        {
            Task.Run(() =>
            {
                appVersion = asReaderP3xUMAUIManager.GetAPPVersion();
                dllVereison = asReaderP3xUMAUIManager.GetAsReaderP3xUSDKDLL();
                sdkVersion = asReaderP3xUMAUIManager.GetSDKVersion();
                firmwareVersion = asReaderP3xUMAUIManager.GetFirmwareVersion();
                rfModuleVersion = asReaderP3xUMAUIManager.GetRFModuleVersion();
                serialVersion = asReaderP3xUMAUIManager.GetSerialNumber();
                globalBandVersion = asReaderP3xUMAUIManager.GetRegion();
                HardwareVersion = asReaderP3xUMAUIManager.GetHardwareNumber();
                MainThread.BeginInvokeOnMainThread(() =>
                {
                    ConnectButton.Text = "Disconnect";
                    APPVersionLabel.Text = appVersion;
                    DLLVersionLabel.Text = dllVereison;
                    SDKVersionLabel.Text = sdkVersion;
                    FirmwareVersionLabel.Text = firmwareVersion;
                    UHFLabel.Text = rfModuleVersion;
                    SNLabel.Text = serialVersion;
                    RegionLabel.Text = globalBandVersion;
                    HardwareLabel.Text = HardwareVersion;
                    RFIDInventoryLabel.TextColor = Colors.Black;
                    LogoImage.Source = "asreader_log_on.png";
                    StopActivityIndicator();
                });
            });
        }
        else
        {
            MainThread.BeginInvokeOnMainThread(() =>
            {
                ConnectButton.Text = "Connect";
                APPVersionLabel.Text = "";
                DLLVersionLabel.Text = "";
                SDKVersionLabel.Text = "";
                FirmwareVersionLabel.Text = "";
                UHFLabel.Text = "";
                SNLabel.Text = "";
                RegionLabel.Text = "";
                HardwareLabel.Text = "";
                RFIDInventoryLabel.TextColor = Colors.LightGray;
                LogoImage.Source = "asreader_log_off.png";
                StopActivityIndicator();
            });
        }
    }
    public void StartActivityIndicator()
    {
        MainActivityIndicator.IsRunning = true;
        ConnectButton.IsEnabled = false;
    }
    public void StopActivityIndicator()
    {
        MainActivityIndicator.IsRunning = false;
        ConnectButton.IsEnabled = true;
    }
    public void ConnectButton_Clicked(object sender, EventArgs e)
    {
        if (!asReaderP3xUMAUIManager.isConnect)
        {
            StartActivityIndicator();
            asReaderP3xUMAUIManager.ConnectDevice();
        }
        else
        {
            StartActivityIndicator();
            asReaderP3xUMAUIManager.DisconnectDevice();
        }
    }
    async void RFIDInventoryClicked(object sender, EventArgs e)
    {
        if (asReaderP3xUMAUIManager.isConnect)
        {
            await Navigation.PushAsync(new RFIDInventoryPage());
        }
    }
}

