﻿using System.Collections;
using System.Collections.ObjectModel;
using AsReaderP3xUAndroidDemo.Platforms.Android;
using Com.Asreader.P3xu;
using Com.Asreader.P3xu.Device.Type;
using Com.Asreader.P3xu.Rfid.Type;

namespace AsReaderP3xUAndroidDemo;

public partial class RFIDInventoryPage : ContentPage
{
    private AsReaderP3xUMAUIManager asReaderP3xUMAUIManager;
    private AsReaderP3xUCallback asReaderP3xUCallback;
    ArrayList powerGainArray;
    int count;
    int powerGain;
    private AsReaderP3xUActionState actionState;
    private ObservableCollection<RFIDData> RFIDItems = new ObservableCollection<RFIDData>();
    public ObservableCollection<RFIDData> ItemData { get { return RFIDItems; } }
    private Dictionary<string, RFIDData> dataDic = new Dictionary<string, RFIDData>();

    public RFIDInventoryPage()
	{
		InitializeComponent();
        count = 0;
        powerGain = 5;
        RFIDActivityIndicator.IsRunning = false;
        actionState = AsReaderP3xUActionState.Stop;
        asReaderP3xUMAUIManager = AsReaderP3xUMAUIManager.sharedInstance();
        asReaderP3xUCallback = AsReaderP3xUCallback.sharedInstance();
    }

    protected override void OnAppearing()
    {
        base.OnAppearing();
        StartActivityIndicator();
        RFIDListView.ItemsSource = RFIDItems;
        asReaderP3xUCallback.OnStateChanged = (AsReaderP3xU p0, AsReaderP3xUConnectionState p1) =>
        {
            OnStateChanged(p0, p1);
        };
        asReaderP3xUCallback.OnKeyEvent = (AsReaderP3xU p0, AsReaderP3xUKeyType p1, AsReaderP3xUKeyState p2) =>
        {
            return OnKeyEvent(p0, p1, p2);
        };
        asReaderP3xUCallback.OnReadTag = (AsReaderP3xU p0, AsReaderP3xUActionState p1, string p2, float p3) =>
        {
            OnReadTag(p0, p1, p2, p3);
        };
        asReaderP3xUCallback.OnActionChanged = (AsReaderP3xU p0, AsReaderP3xUActionState p1) =>
        {
            OnActionChanged(p0, p1);
        };
        asReaderP3xUCallback.OnReceivedDidBecomeActive = (bool status) =>
        {
            OnReceivedDidBecomeActive(status);
        };
        Task.Run(() =>
        {
            try
            {
                powerGainArray = asReaderP3xUMAUIManager.GetPowerGainRange();
                Thread.Sleep(1000);
                powerGain = asReaderP3xUMAUIManager.PowerGain;
            }
            catch (Exception)
            {
                powerGain = asReaderP3xUMAUIManager.GetPowerRangeMax();
            }
            try
            {
                asReaderP3xUMAUIManager.SetContinuousMode(true);
            }
            catch (Exception)
            {
            }
            MainThread.BeginInvokeOnMainThread(() =>
            {
                RescanTimePicker.Title = (powerGain / 10).ToString() + ".0 dBm";
                if (powerGainArray != null)
                {
                    foreach (string power in powerGainArray)
                    {
                        RescanTimePicker.Items.Add(power + ".0 dBm");
                    }
                }
                StopActivityIndicator();
            });
        });
    }
    protected override void OnDisappearing()
    {
        base.OnDisappearing();
        asReaderP3xUMAUIManager.Stop();
    }
    void StartScanButton_Clicked(object sender, EventArgs e)
    {
        if (actionState == AsReaderP3xUActionState.Inventory)
        {
            asReaderP3xUMAUIManager.Stop();
        }
        else if (actionState == AsReaderP3xUActionState.Stop)
        {
            asReaderP3xUMAUIManager.Inventory();
        }
    }
    void ClearScanButton_Clicked(object sender, EventArgs e)
    {
        count = 0;
        CountLabel.Text = count.ToString();
        dataDic.Clear();
        RFIDItems.Clear();
    }
    void OnPickerSelectedIndexChanged(object sender, EventArgs e)
    {
        var picker = (Picker)sender;
        int selectedIndex = picker.SelectedIndex;
        if (selectedIndex != -1)
        {
            string powerGain = picker.Items[selectedIndex];
            RescanTimePicker.Title = powerGain;
            powerGain = powerGain.Replace(".0 dBm", ""); 
            asReaderP3xUMAUIManager.PowerGain = int.Parse(powerGain) * 10;
        }
    }
    private void OnStateChanged(AsReaderP3xU p0, AsReaderP3xUConnectionState p1)
    {
        if (p1 != AsReaderP3xUConnectionState.Connected)
        {
            MainThread.BeginInvokeOnMainThread(() =>
            {
                Navigation.PopAsync();
            });
        }
    }
    private void OnReadTag(AsReaderP3xU p0, AsReaderP3xUActionState p1, string p2, float p3)
    {
        RFIDData InventoryDataChanged;
        if (dataDic.ContainsKey(p2))
        {
            InventoryDataChanged = dataDic[p2];
            InventoryDataChanged.Count++;
            dataDic[p2] = InventoryDataChanged;
        }
        else
        {
            int nowCount = dataDic.Count;
            InventoryDataChanged = new RFIDData(p2, 1);
            dataDic.Add(p2, InventoryDataChanged);
            RFIDItems.Add(InventoryDataChanged);
        }
        count++;
        MainThread.BeginInvokeOnMainThread(() =>
        {
            RFIDListView.ItemsSource = null;
            RFIDListView.ItemsSource = RFIDItems;
            CountLabel.Text = count.ToString();
        });
    }
    private bool OnKeyEvent(AsReaderP3xU p0, AsReaderP3xUKeyType p1, AsReaderP3xUKeyState p2)
    {
        if (p1 == AsReaderP3xUKeyType.Trigger)
        {
            if (p2 == AsReaderP3xUKeyState.KeyDown)
            {
                asReaderP3xUMAUIManager.Inventory();
            }
            else
            {
                asReaderP3xUMAUIManager.Stop();
            }
        }
        return false;
    }
    private void OnActionChanged(AsReaderP3xU p0, AsReaderP3xUActionState p1)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            if (p1 == AsReaderP3xUActionState.Inventory)
            {
                actionState = AsReaderP3xUActionState.Inventory;
                StartScanButton.Text = "Stop Scan";
            }
            else if (p1 == AsReaderP3xUActionState.Stop)
            {
                actionState = AsReaderP3xUActionState.Stop;
                StartScanButton.Text = "Inventory";
            }
        });
    }
    private void OnReceivedDidBecomeActive(bool status)
    {
        try
        {
            if (status)
            {
                //asReaderP3xUMAUIManager.Inventory();
            }
            else
            {
                asReaderP3xUMAUIManager.Stop();
            }
        }
        catch { }
    }
    public void StartActivityIndicator()
    {
        RFIDActivityIndicator.IsRunning = true;
        RFIDListView.IsEnabled = false;
        StartScanButton.IsEnabled = false;
        ClearScanButton.IsEnabled = false;
        RescanTimePicker.IsEnabled = false;
    }
    public void StopActivityIndicator()
    {
        RFIDActivityIndicator.IsRunning = false;
        RFIDListView.IsEnabled = true;
        StartScanButton.IsEnabled = true;
        ClearScanButton.IsEnabled = true;
        RescanTimePicker.IsEnabled = true;
    }
}
